//
//  Huffman Code 1.0
//  HuffmanDecoder.cpp
//
//  Created by Amittai Aviram on 2 Sep 2018.
//  Copyright © 2018 Amittai Aviram. All rights reserved.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

#include "HuffmanDecoder.hpp"
#include <fstream>
#include <iostream>


HuffmanDecoder::~HuffmanDecoder()
{
    for (auto node : tree_nodes)
    {
        delete node;
    }
}

void HuffmanDecoder::readMessageFromFileAndDecode(const std::string& input_file, const std::string& output_file)
{
    readMessageFromFileAndDecode(input_file);
    std::ofstream output(output_file);
    output << decoded_message;
}

void HuffmanDecoder::readMessageFromFileAndDecode(const std::string& input_file)
{
    readMessageFromFile(input_file);
    decodeFromFile();
}

void HuffmanDecoder::readMessageFromFile(const std::string& file_name)
{
    std::ifstream input(file_name, std::ifstream::in | std::ifstream::binary);
    input.read(reinterpret_cast<char *>(&tree_index), sizeof(size_t));
    std::size_t nodes_size = 0U;
    input.read(reinterpret_cast<char *>(& nodes_size), sizeof(size_t));
    for (size_t i = 0; i < nodes_size; ++i)
    {
        TreeNode * new_node = new TreeNode();
        input.read(reinterpret_cast<char *>(new_node), sizeof(TreeNode));
        tree_nodes.push_back(new_node);
    }
    std::size_t encoded_message_size = 0U;
    input.read(reinterpret_cast<char *>(&encoded_message_size), sizeof(size_t));
    uint64_t num = 0ULL;
    for (size_t i = 0; i < encoded_message_size; ++i)
    {
        input.read(reinterpret_cast<char *>(&num), sizeof(uint64_t));
        encoded_message.push_back(num);
    }
    tree = tree_nodes[tree_index];
}

void HuffmanDecoder::decodeFromFile()
{
    int encoded_message_index = 0;
    int bit_index = 0;
    std::bitset<64> entry{encoded_message[encoded_message_index]};
    TreeNode * current_node = tree;
    char current_char = 'a';
    while (current_char != '\0')
    {
        while (!current_node->isLeaf())
        {
            if (entry[bit_index])
            {
                current_node = tree_nodes[current_node->right_index];
            }
            else
            {
                current_node = tree_nodes[current_node->left_index];
            }
            ++bit_index;
            if (bit_index >= 64)
            {
                ++encoded_message_index;
                entry = std::bitset<64>{encoded_message[encoded_message_index]};
                if (encoded_message_index >= encoded_message.size())
                {
                    break;
                }
                bit_index = 0;
            }
        }
        current_char = current_node->key;
        if (current_char == '\0')
        {
            break;
        }
        decoded_message += current_char;
        current_node = tree;
    }
}

std::string& HuffmanDecoder::decodedMessage()
{
    return decoded_message;
}
