//
//  Huffman Code 1.0
//  Encode.cpp
//
//  Created by Amittai Aviram on 2 Sep 2018.
//  Copyright © 2018 Amittai Aviram. All rights reserved.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

#include "HuffmanEncoder.hpp"
#include <cstdlib>
#include <cstring>
#include <iostream>
#include <string>

void usage(char ** argv)
{
    std::cerr << "Usage:\n";
    std::cerr << argv[0] << " -f <input_file> <output_file>\n";
    std::cerr << argv[0] << "\"<input_text>\"" << std::endl;
}

int main(int argc, char ** argv)
{
    if (argc < 4)
    {
        usage(argv);
        exit(EXIT_FAILURE);
    }

    std::cout << "Huffman Encoder" << std::endl;
    HuffmanEncoder encoder;
    const std::string output_file(argv[3]);

    if (!strcmp(argv[1], "-f"))
    {
        const std::string input_file(argv[2]);
        std::cout << "Reading text from file " << input_file << " and\n";
        std::cout << "saving the encoded version in " << output_file << " ..." << std::endl;
        encoder.encodeMessageAndWriteToFile(input_file, output_file);
    }
    else
    {
        const std::string input_text(argv[2]);
        std::cout << "Saving encoded version of input text in " << output_file << " ..." << std::endl;
        encoder.setInputMessage(input_text);
    }

    return EXIT_SUCCESS;
}
